@@ -16,13 +16,14 @@ from .const import (
     DEFAULT_PRINTER_ATTRIBUTES,
     DEFAULT_PROTO_VERSION,
 )
-from .enums import IppOperation
+from .enums import IppOperation, IppStatus
 from .exceptions import (
     IPPConnectionError,
     IPPConnectionUpgradeRequired,
     IPPError,
     IPPParseError,
     IPPResponseError,
+    IPPVersionNotSupportedError,
 )
 from .models import Printer
 from .parser import parse as parse_response
@@ -100,9 +101,6 @@ class IPP:
             self._session = aiohttp.ClientSession()
             self._close_session = True
 
-        if data is None:
-            data = {}
-
         if isinstance(data, dict):
             data = encode_dict(data)
 
@@ -117,14 +115,14 @@ class IPP:
                     headers=headers,
                     ssl=self.verify_ssl,
                 )
-        except asyncio.TimeoutError as exception:
+        except asyncio.TimeoutError as exc:
             raise IPPConnectionError(
                 "Timeout occurred while connecting to IPP server."
-            ) from exception
-        except (aiohttp.ClientError, SocketGIAError) as exception:
+            ) from exc
+        except (aiohttp.ClientError, SocketGIAError) as exc:
             raise IPPConnectionError(
                 "Error occurred while communicating with IPP server."
-            ) from exception
+            ) from exc
 
         if response.status == 426:
             raise IPPConnectionUpgradeRequired(
@@ -180,10 +178,13 @@ class IPP:
 
         try:
             parsed = parse_response(response)
-        except (StructError, Exception) as exception:  # disable=broad-except
-            raise IPPParseError from exception
+        except (StructError, Exception) as exc:  # disable=broad-except
+            raise IPPParseError from exc
 
-        if parsed["status-code"] != 0:
+        if parsed["status-code"] == IppStatus.ERROR_VERSION_NOT_SUPPORTED:
+            raise IPPVersionNotSupportedError("IPP version not supported by server")
+
+        if parsed["status-code"] != IppStatus.OK:
             raise IPPError(
                 "Unexpected printer status code",
                 {"status-code": parsed["status-code"]},
@@ -213,8 +214,14 @@ class IPP:
             },
         )
 
-        data: dict = next(iter(response_data["printers"] or []), {})
-        return Printer.from_dict(data)
+        parsed: dict = next(iter(response_data["printers"] or []), {})
+
+        try:
+            printer = Printer.from_dict(parsed)
+        except Exception as exc:  # disable=broad-except
+            raise IPPParseError from exc
+
+        return printer
 
     async def __aenter__(self) -> "IPP":
         """Async enter."""
